@@ -13,8 +13,9 @@
 # permissions and limitations under the License.
 
 """Support for SSL in PyMongo."""
+from __future__ import annotations
 
-import sys
+from typing import Optional
 
 from pymongo.errors import ConfigurationError
 
@@ -34,31 +35,30 @@ if HAVE_SSL:
     # CPython ssl module constants to configure certificate verification
     # at a high level. This is legacy behavior, but requires us to
     # import the ssl module even if we're only using it for this purpose.
-    import ssl as _stdlibssl  # noqa
+    import ssl as _stdlibssl  # noqa: F401
     from ssl import CERT_NONE, CERT_REQUIRED
 
     HAS_SNI = _ssl.HAS_SNI
-    IPADDR_SAFE = _ssl.IS_PYOPENSSL or sys.version_info[:2] >= (3, 7)
+    IPADDR_SAFE = True
     SSLError = _ssl.SSLError
+    BLOCKING_IO_ERRORS = _ssl.BLOCKING_IO_ERRORS
 
     def get_ssl_context(
-        certfile,
-        passphrase,
-        ca_certs,
-        crlfile,
-        allow_invalid_certificates,
-        allow_invalid_hostnames,
-        disable_ocsp_endpoint_check,
-    ):
+        certfile: Optional[str],
+        passphrase: Optional[str],
+        ca_certs: Optional[str],
+        crlfile: Optional[str],
+        allow_invalid_certificates: bool,
+        allow_invalid_hostnames: bool,
+        disable_ocsp_endpoint_check: bool,
+    ) -> _ssl.SSLContext:
         """Create and return an SSLContext object."""
         verify_mode = CERT_NONE if allow_invalid_certificates else CERT_REQUIRED
         ctx = _ssl.SSLContext(_ssl.PROTOCOL_SSLv23)
-        # SSLContext.check_hostname was added in CPython 3.4.
-        if hasattr(ctx, "check_hostname"):
-            if _ssl.CHECK_HOSTNAME_SAFE and verify_mode != CERT_NONE:
-                ctx.check_hostname = not allow_invalid_hostnames
-            else:
-                ctx.check_hostname = False
+        if verify_mode != CERT_NONE:
+            ctx.check_hostname = not allow_invalid_hostnames
+        else:
+            ctx.check_hostname = False
         if hasattr(ctx, "check_ocsp_endpoint"):
             ctx.check_ocsp_endpoint = not disable_ocsp_endpoint_check
         if hasattr(ctx, "options"):
@@ -74,12 +74,14 @@ if HAVE_SSL:
             try:
                 ctx.load_cert_chain(certfile, None, passphrase)
             except _ssl.SSLError as exc:
-                raise ConfigurationError("Private key doesn't match certificate: %s" % (exc,))
+                raise ConfigurationError(f"Private key doesn't match certificate: {exc}") from None
         if crlfile is not None:
             if _ssl.IS_PYOPENSSL:
                 raise ConfigurationError("tlsCRLFile cannot be used with PyOpenSSL")
             # Match the server's behavior.
-            setattr(ctx, "verify_flags", getattr(_ssl, "VERIFY_CRL_CHECK_LEAF", 0))  # noqa
+            ctx.verify_flags = getattr(  # type:ignore[attr-defined]
+                _ssl, "VERIFY_CRL_CHECK_LEAF", 0
+            )
             ctx.load_verify_locations(crlfile)
         if ca_certs is not None:
             ctx.load_verify_locations(ca_certs)
@@ -95,6 +97,7 @@ else:
 
     HAS_SNI = False
     IPADDR_SAFE = False
+    BLOCKING_IO_ERRORS = ()  # type:ignore[assignment]
 
     def get_ssl_context(*dummy):  # type: ignore
         """No ssl module, raise ConfigurationError."""
