@@ -16,6 +16,7 @@
 from __future__ import annotations
 
 import os
+import socketserver
 import sys
 import threading
 
@@ -27,6 +28,7 @@ from test.unified_format import generate_test_classes
 from test.utils import (
     CMAPListener,
     HeartbeatEventListener,
+    HeartbeatEventsListListener,
     assertion_context,
     client_context,
     get_pool,
@@ -38,7 +40,7 @@ from test.utils import (
 from unittest.mock import patch
 
 from bson import Timestamp, json_util
-from pymongo import common, monitoring
+from pymongo import MongoClient, common, monitoring
 from pymongo.errors import (
     AutoReconnect,
     ConfigurationError,
@@ -48,6 +50,7 @@ from pymongo.errors import (
 )
 from pymongo.hello import Hello, HelloCompat
 from pymongo.helpers import _check_command_response, _check_write_command_response
+from pymongo.monitoring import ServerHeartbeatFailedEvent, ServerHeartbeatStartedEvent
 from pymongo.server_description import SERVER_TYPE, ServerDescription
 from pymongo.settings import TopologySettings
 from pymongo.topology import Topology, _ErrorContext
@@ -396,6 +399,48 @@ class TestServerMonitoringMode(IntegrationTest):
             self.assertIsNone(monitor._rtt_monitor._executor._thread)
 
 
+class MockTCPHandler(socketserver.BaseRequestHandler):
+    def handle(self):
+        self.server.events.append("client connected")
+        if self.request.recv(1024).strip():
+            self.server.events.append("client hello received")
+        self.request.close()
+
+
+class TCPServer(socketserver.TCPServer):
+    allow_reuse_address = True
+
+    def handle_request_and_shutdown(self):
+        self.handle_request()
+        self.server_close()
+
+
+class TestHeartbeatStartOrdering(unittest.TestCase):
+    def test_heartbeat_start_ordering(self):
+        events = []
+        listener = HeartbeatEventsListListener(events)
+        server = TCPServer(("localhost", 9999), MockTCPHandler)
+        server.events = events
+        server_thread = threading.Thread(target=server.handle_request_and_shutdown)
+        server_thread.start()
+        _c = MongoClient(
+            "mongodb://localhost:9999", serverSelectionTimeoutMS=500, event_listeners=(listener,)
+        )
+        server_thread.join()
+        listener.wait_for_event(ServerHeartbeatStartedEvent, 1)
+        listener.wait_for_event(ServerHeartbeatFailedEvent, 1)
+
+        self.assertEqual(
+            events,
+            [
+                "serverHeartbeatStartedEvent",
+                "client connected",
+                "client hello received",
+                "serverHeartbeatFailedEvent",
+            ],
+        )
+
+
 # Generate unified tests.
 globals().update(generate_test_classes(os.path.join(SDAM_PATH, "unified"), module=__name__))
 
