@@ -13,23 +13,25 @@
 # limitations under the License.
 
 """Test the server module."""
+from __future__ import annotations
 
 import sys
 
 sys.path[0:0] = [""]
 
-from pymongo.ismaster import IsMaster
+from test import unittest
+
+from pymongo.hello import Hello
 from pymongo.server import Server
 from pymongo.server_description import ServerDescription
-from test import unittest
 
 
 class TestServer(unittest.TestCase):
     def test_repr(self):
-        ismaster = IsMaster({'ok': 1})
-        sd = ServerDescription(('localhost', 27017), ismaster)
-        server = Server(sd, pool=object(), monitor=object())
-        self.assertTrue('Standalone' in str(server))
+        hello = Hello({"ok": 1})
+        sd = ServerDescription(("localhost", 27017), hello)
+        server = Server(sd, pool=object(), monitor=object())  # type: ignore[arg-type]
+        self.assertTrue("Standalone" in str(server))
 
 
 if __name__ == "__main__":
