@@ -18,6 +18,7 @@ from __future__ import annotations
 import copy
 import sys
 import warnings
+from typing import Any
 from urllib.parse import quote_plus
 
 sys.path[0:0] = [""]
@@ -27,7 +28,12 @@ from test import unittest
 from bson.binary import JAVA_LEGACY
 from pymongo import ReadPreference
 from pymongo.errors import ConfigurationError, InvalidURI
-from pymongo.uri_parser import parse_uri, parse_userinfo, split_hosts, split_options
+from pymongo.uri_parser import (
+    parse_uri,
+    parse_userinfo,
+    split_hosts,
+    split_options,
+)
 
 
 class TestURI(unittest.TestCase):
@@ -143,6 +149,12 @@ class TestURI(unittest.TestCase):
         self.assertEqual({"authsource": "foobar"}, split_options("authSource=foobar"))
         self.assertEqual({"maxpoolsize": 50}, split_options("maxpoolsize=50"))
 
+        # Test suggestions given when invalid kwarg passed
+
+        expected = r"Unknown option: auth. Did you mean one of \(authsource, authmechanism, timeoutms\) or maybe a camelCase version of one\? Refer to docstring."
+        with self.assertRaisesRegex(ConfigurationError, expected):
+            split_options("auth=GSSAPI")
+
     def test_parse_uri(self):
         self.assertRaises(InvalidURI, parse_uri, "http://foobar.com")
         self.assertRaises(InvalidURI, parse_uri, "http://foo@foobar.com")
@@ -470,7 +482,7 @@ class TestURI(unittest.TestCase):
             "&authMechanismProperties=AWS_SESSION_TOKEN:" + quoted_val
         )
         res = parse_uri(uri)
-        options = {
+        options: dict[str, Any] = {
             "authmechanism": "MONGODB-AWS",
             "authmechanismproperties": {"AWS_SESSION_TOKEN": unquoted_val},
         }
@@ -492,20 +504,30 @@ class TestURI(unittest.TestCase):
         self.assertEqual(options, res["options"])
 
     def test_redact_AWS_SESSION_TOKEN(self):
-        unquoted_colon = "token:"
+        token = "token"
         uri = (
             "mongodb://user:password@localhost/?authMechanism=MONGODB-AWS"
-            "&authMechanismProperties=AWS_SESSION_TOKEN:" + unquoted_colon
+            "&authMechanismProperties=AWS_SESSION_TOKEN-" + token
         )
         with self.assertRaisesRegex(
             ValueError,
-            "auth mechanism properties must be key:value pairs like "
-            "SERVICE_NAME:mongodb, not AWS_SESSION_TOKEN:<redacted token>"
-            ", did you forget to percent-escape the token with "
-            "quote_plus?",
+            "auth mechanism properties must be key:value pairs like AWS_SESSION_TOKEN:<token>",
         ):
             parse_uri(uri)
 
+    def test_handle_colon(self):
+        token = "token:foo"
+        uri = (
+            "mongodb://user:password@localhost/?authMechanism=MONGODB-AWS"
+            "&authMechanismProperties=AWS_SESSION_TOKEN:" + token
+        )
+        res = parse_uri(uri)
+        options = {
+            "authmechanism": "MONGODB-AWS",
+            "authMechanismProperties": {"AWS_SESSION_TOKEN": token},
+        }
+        self.assertEqual(options, res["options"])
+
     def test_special_chars(self):
         user = "user@ /9+:?~!$&'()*+,;="
         pwd = "pwd@ /9+:?~!$&'()*+,;="
